<?php
// +---------------------------------------------------------------------------+
// |                           Ultimate FTP Class                              |
// +---------------------------------------------------------------------------+
// | Advanced FTP wrapper for handling FTP connections and file                |
// | sharing/remoting via FTP.                                                 |
// +---------------------------------------------------------------------------+
// |  Written by Vladimir Sibirov a.k.a. Trustmaster.                          |
// |  e-mail: trustware@bk.ru                                                  |
// |  Sorry, no web links here.                                                |
// +---------------------------------------------------------------------------+

/**
* This class provides methods for handling FTP connections. It is not only an object
* oriented wrapper to standard PHP ftp_ functions but also provides some routines for
* advanced file sharing/remoting via FTP.
* We're looking forward to stable PHP5 which gives some more advantages for handling
* FTP protocol in PHP.
* Sorry, no examples this time, but the logic is rather clear. See also the PHP Manual.
*
* @access public
* @package FTP Class
* @version FTP Class, v.1.0.0
* @author Vladimir Sibirov <trustware@bk.ru>
*/
class FTP
{
	/**
	* FTP connection identifier.
	*/
	var $stream = null;
	/**
	* Server name or IP.
	*/
	var $host = 'localhost';
	/**
	* FTP service port number.
	*/
	var $port = 21;
	/**
	* FTP account username.
	*/
	var $user = 'anonymous';
	/**
	* FTP account password
	*/
	var $pass = 'anonymous';
	/**
	* Connection security (SSL)
	*/
	var $secure = false;
	/**
	* Connection passive mode.
	*/
	var $passive = false;
	
	/**
	* Constructor: connects to the FTP server and creates new FTP object associated with the connection.
	*
	* @access public
	* @return FTP
	* @param string $host FTP server's hostname.
	* @param bool $use_ssl Wheter to use secure SSL connection.
	* @param int $port FTP service port number.
	*/
	function FTP($host = 'localhost', $use_ssl = false, $port = 21)
	{
		if(!defined('FTP_TEMP_DIRECTORY'))
		{
			if(strtoupper(PHP_OS) == 'WINDOWS') define('FTP_TEMP_DIRECTORY', 'C:/Windows/temp');
			else define('FTP_TEMP_DIRECTORY', '/tmp');
		}
		if($host != 'localhost') $this->host = $host;
		$this->port = $port;
		$this->secure = $use_ssl;
		$connect_func = ($this->secure) ? 'ftp_ssl_connect' : 'ftp_connect';
		$this->stream = $connect_func($this->host, $this->port)
		or die('FTP Error: Couldn\'t connect to server.');
		if(defined('FTP_PASSIVE')) $this->passive = FTP_PASSIVE;
		if($this->passive) @ftp_pasv($this->stream, $this->passive);
	}
	
	/**
	* Logs in as a custom user.
	*
	* @access public
	* @return bool
	* @param string $username FTP account username.
	* @param string $password FTP account password.
	*/
	function Login($username = 'anonymous', $password = 'anonymous')
	{
		$this->user = $username;
		$this->pass = $password;
		return @ftp_login($this->stream, $this->user, $this->pass);
	}
	
	/**
	* If mode is defined then turns data transfer passive mode to its value. Otherwise returns
	* current value of passive mode.
	* In passive mode, data connections are initiated by the client, rather than by the server.
	*
	* @access public
	* @return bool
	* @param bool $mode Whether to switch transfer mode to passive.
	*/
	function Passive($mode = null)
	{
		if(!is_null($mode))
		{
			$this->passive = $mode;
			return @ftp_pasv($this->stream, $this->passive);
		}
		else return $this->passive;
	}
	
	/**
	* Returns value of specified FTP option.
	*
	* @access public
	* @return mixed
	* @param int $option Server option.
	*/
	function GetOption($option)
	{
		return @ftp_get_option($this->stream, $option);
	}
	
	/**
	* Sets specified FTP option.
	*
	* @access public
	* @return bool
	* @param int $option Server option.
	* @param mixed $value New value.
	*/
	function SetOption($option, $value)
	{
		return @ftp_set_option($this->stream, $option, $value);
	}
	
	/**
	* Returns the current directory name, or false on error.
	*
	* @access public
	* @return string
	*/
	function Pwd()
	{
		return @ftp_pwd($this->stream);
	}
	
	/**
	* Changes to the parent directory.
	*
	* @access public
	* @return bool
	*/
	function Cdup()
	{
		return @ftp_cdup($this->stream);
	}
	
	/**
	* Changes to the specified directory.
	* 
	* @access public
	* @return bool
	* @param string $directory Destination directory.
	*/
	function Chdir($directory)
	{
		return @ftp_chdir($this->stream, $directory);
	}
	
	/**
	* Sets the permissions on the remote file specified by filename to mode given as
	* an octal value. Returns the new file permissions on success or FALSE on error.
	*
	* @access public
	* @return string
	* @param int $mode Access mode in octal format. e.g. 0644.
	* @param string $filename Remote file name.
	*/
	function Chmod($mode, $filename)
	{
		return @ftp_chmod($this->stream, $mode, $filename);
	}
	
	/**
	* Creates the specified directory. Returns the newly created directory name on success, false on error.
	*
	* @access public
	* @return string
	* @param string $directory Remote directory name.
	*/
	function Mkdir($directory)
	{
		return ftp_mkdir($this->stream, $directory);
	}
	
	/**
	* Removes the remote directory even if non-empty. In this case the directory is removed
	* recursively.
	*
	* @access public
	* @return bool
	* @param string $directory Remote directory.
	*/
	function Rmdir($directory)
	{
		$directory = str_replace('\\', '/', $directory);
		$raw = $this->RawList($directory);
		for($i = 0; $i < count($raw); $i++)
		{
			$file = $this->ExtractInfo($raw[$i]);
			if(!empty($file['name']) && ($file['name'] != '.') && ($file['name'] != '..'))
			{ 
				if(ereg('^drw', $file['mode'])) $this->Rmdir($directory.'/'.$file['name']);
				else $this->Delete($directory.'/'.$file['name']);
			}
		}
		return @ftp_rmdir($this->stream, $directory);
	}
	
	/**
	* Returns an array of filenames on success, false on error.
	*
	* @access public
	* @return array
	* @param string $directory Target directory.
	*/
	function Directory($directory = null)
	{
		if(is_null($directory)) $directory = $this->Pwd();
		return @ftp_nlist($this->stream, $directory);
	}
	
	/**
	* Executes the FTP LIST command, and returns the result as an array.
	* Each array element corresponds to one line of text. The output is not parsed in any way.
	* You can extract details for each element using ExtractInfo() method.
	*
	* @access public
	* @return array
	* @param string $directory Remote directory name.
	*/
	function RawList($directory = null)
	{
		if(is_null($directory)) $directory = $this->Pwd();
		return @ftp_rawlist($this->stream, $directory);
	}
	
	/**
	* This function extracts file information from the raw result string. Associative array
	* containing file properties is returned. Here are property names and example values:
	* ['raw'] : exactly matches $raw_data.
	* ['name'] : file name, e.g. 'filename.html' or 'Some Folder'.
	* ['mode'] : access mode in string representation, e.g. 'drw-rw-rw-' or '-rwxr-x-r-x'.
	* ['size'] : file size in bytes, e.g. 1024.
	* ['id'] : sorry, haven't understood yet what this identifier means.
	* ['user'] : file owner, e.g. 'root'.
	* ['user'] : owner's group, e.g. 'wheel'.
	* ['date'] : last modification date, e.g. 'Sep 11 22:13'.
	* ['mon'] : modification month, e.g. 'Feb'.
	* ['day'] : modification day of the month, e.g. 28.
	* ['time'] : modification time, or sometimes it contains year, e.g. '18:12' or '2004'.
	*
	* @access public
	* @return array
	* @param string $raw_data A record from RawList() result array.
	*/
	function ExtractInfo($raw_data)
	{
		ereg('^([a-z\-]{10})( +)([0-9]+)( +)([a-zA-Z0-9_]+)( +)([a-zA-Z0-9_]+)( +)([0-9]+) (((Jan)|(Feb)|(Mar)|(Apr)|(May)|(Jun)|(Jul)|(Aug)|(Sep)|(Oct)|(Nov)|(Dec))( {1,2})([0-9]+)( {1,2})([0-9:]+)) ([^/\\:?*<>"]+)$', $raw_data, $regs);
		$file['raw'] = $regs[0];
		$file['mode'] = $regs[1];
		$file['id'] = $regs[3];
		$file['user'] = $regs[5];
		$file['group'] = $regs[7];
		$file['size'] = $regs[9];
		$file['date'] = $regs[10];
		$file['mon'] = $regs[11];
		$file['day'] = $regs[25];
		$file['time'] = $regs[27];
		$file['name'] = $regs[28];
		return $file;
	}
	
	/**
	* Returns the system type (e.g. "UNIX") of the remote FTP server.
	*
	* @access public
	* @return string
	*/
	function SysType()
	{
		return @ftp_systype($this->stream);
	}
	
	/**
	* Retrieves remote_file from the FTP server, and saves it to local_file locally.
	* The transfer mode specified must be either FTP_ASCII or FTP_BINARY.
	*
	* @access public
	* @return bool
	* @param string $local_file Local file name (destination).
	* @param string $remote_file Remote file name (source).
	* @param int $mode Transfer mode: FTP_ASCII for text files or FTP_BINARY for others.
	*/
	function Get($local_file, $remote_file, $mode = FTP_BINARY)
	{
		return @ftp_get($this->stream, $local_file, $remote_file, $mode);
	}
	
	/**
	* Retrieves remote_file from the FTP server, and saves it to local_file locally.
	* The transfer mode specified must be either FTP_ASCII or FTP_BINARY.
	* The difference with Get() method is that this function retrieves the file
	* asynchronously, so your program can perform other operations while the file is
	* being downloaded.
	*
	* @access public
	* @return bool
	* @param string $local_file Local file name (destination).
	* @param string $remote_file Remote file name (source).
	* @param int $mode Transfer mode: FTP_ASCII for text files or FTP_BINARY for others.
	*/
	function GetNb($local_file, $remote_file, $mode = FTP_BINARY)
	{
		return @ftp_nb_get($this->stream, $local_file, $remote_file, $mode);
	}
	
	/**
	* Retrieves remote_file from the FTP server, and writes it to the given file pointer, fp.
	* The transfer mode specified must be either FTP_ASCII or FTP_BINARY.
	*
	* @access public
	* @return resource
	* @param string $filename Remote file name.
	* @param int $mode Transfer mode: FTP_ASCII for text files or FTP_BINARY for others.
	*/
	function FGet($filename, $mode = FTP_ASCII)
	{
		$fp = null;
		$result = @ftp_fget($this->stream, $fp, $filename, $mode);
		if($result) return $fp;
		else return false;
	}
	
	/**
	* Retrieves remote_file from the FTP server, and writes it to the given file pointer, fp.
	* The transfer mode specified must be either FTP_ASCII or FTP_BINARY.
	* The difference with FGet() method is that this function retrieves the file
	* asynchronously, so your program can perform other operations while the file is
	* being downloaded.
	*
	* @access public
	* @return resource
	* @param string $filename Remote file name.
	* @param int $mode Transfer mode: FTP_ASCII for text files or FTP_BINARY for others.
	*/
	function FGetNb($filename, $mode = FTP_ASCII)
	{
		$fp = null;
		$result = @ftp_nb_fget($this->stream, $fp, $filename, $mode);
		if($result) return $fp;
		else return false;
	}
	
	/**
	* Downloads full directory recursively.
	*
	* @access public
	* @return bool
	* @param string $local_path Destination directory (local).
	* @param string $remote_path Source directory (remote).
	*/
	function GetDirectory($local_path, $remote_path)
	{
		$local_path = str_replace('\\', '/', $local_path);
		if(!file_exists($local_path)) mkdir($local_path);
		$raw = $this->RawList($remote_path);
		for($i = 0; $i < count($raw); $i++)
		{
			$file = $this->ExtractInfo($raw[$i]);
			if(!empty($file['name']) && ($file['name'] != '.') && ($file['name'] != '..'))
			{ 
				if(ereg('^drw', $file['mode'])) $this->GetDirectory($local_path.'/'.$file['name'], $remote_path.'/'.$file['name']);
				else $this->Get($local_path.'/'.$file['name'], $remote_path.'/'.$file['name'], FTP_BINARY);
			}
		}
		return true;
	}
	
	/**
	* Sends an ALLO command to the remote FTP server to allocate filesize bytes of space.
	* Returns string containing the server's responses on success, or FALSE on failure.
	* Note: Many FTP servers do not support this command. These servers may return a
	* failure code (FALSE) indicating the command is not supported or a success code
	* (TRUE) to indicate that pre-allocation is not necessary and the client should continue
	* as though the operation were successful. Because of this, it may be best to reserve this
	* function for servers which explicitly require preallocation.
	* 
	* @access public
	* @return string
	* @param int $filesize Size of the file for which we're allocating free space.
	*/
	function Alloc($filesize)
	{
		$result = null;
		if(@ftp_alloc($this->stream, $filesize, $result)) return $result;
		else return false;
	}
	
	/**
	* Stores local_file on the FTP server, as remote_file.
	* The transfer mode specified must be either FTP_ASCII or FTP_BINARY.
	*
	* @access public
	* @return bool
	* @param string $remote_file Remote file name (destination).
	* @param string $local_file Local file name (source).
	* @param int $mode Transfer mode: FTP_ASCII for text files or FTP_BINARY for others.
	*/
	function Put($remote_file, $local_file, $mode = FTP_BINARY)
	{
		return @ftp_put($this->stream, $remote_file, $local_file, $mode);
	}
	
	/**
	* Stores local_file on the FTP server, as remote_file.
	* The transfer mode specified must be either FTP_ASCII or FTP_BINARY.
	* The difference between this method and the Put() is that this function uploads the file
	* asynchronously, so your program can perform other operations while the file is being uploaded.
	*
	* @access public
	* @return bool
	* @param string $remote_file Remote file name (destination).
	* @param string $local_file Local file name (source).
	* @param int $mode Transfer mode: FTP_ASCII for text files or FTP_BINARY for others.
	*/
	function PutNb($remote_file, $local_file, $mode = FTP_BINARY)
	{
		return @ftp_nb_put($this->stream, $remote_file, $local_file, $mode);
	}
	
	/**
	* Uploads the data from the file pointer fp until end of file. The results are stored in remote_file on the FTP server.
	* The transfer mode specified must be either FTP_ASCII or FTP_BINARY.
	*
	* @access public
	* @return bool
	* @param string $remote_file Remote file name (destination).
	* @param resource $fp File pointer.
	* @param int $mode Transfer mode: FTP_ASCII for text files or FTP_BINARY for others.
	*/
	function FPut($remote_file, $fp, $mode = FTP_BINARY)
	{
		return @ftp_fput($this->stream, $remote_file, $fp, $mode);
	}
	
	/**
	* Uploads the data from the file pointer fp until end of file. The results are stored in remote_file on the FTP server.
	* The transfer mode specified must be either FTP_ASCII or FTP_BINARY.
	* The difference between this method and the Put() is that this function uploads the file
	* asynchronously, so your program can perform other operations while the file is being uploaded.
	*
	* @access public
	* @return bool
	* @param string $remote_file Remote file name (destination).
	* @param resource $fp File pointer.
	* @param int $mode Transfer mode: FTP_ASCII for text files or FTP_BINARY for others.
	*/
	function FPutNb($remote_file, $fp, $mode = FTP_BINARY)
	{
		return @ftp_nb_fput($this->stream, $remote_file, $fp, $mode);
	}
	
	/**
	* Uploads full directory recursively.
	*
	* @access public
	* @return bool
	* @param string $local_path Destination directory (local).
	* @param string $remote_path Source directory (remote).
	*/
	function PutDirectory($remote_path, $local_path)
	{
		$local_path = str_replace('\\', '/', $local_path);
		$this->Mkdir($remote_path);
		$dp = opendir($local_path);
		while($filename = readdir($dp))
		{
			if(($filename != '.') && ($filename != '..'))
			{ 
				if(is_dir($local_path.'/'.$filename)) $this->PutDirectory($remote_path.'/'.$filename, $local_path.'/'.$filename);
				else $this->Put($remote_path.'/'.$filename, $local_path.'/'.$filename, FTP_BINARY);
			}
		}
		closedir($dp);
		return true;
	}
	
	/**
	* Continues retrieving/sending a file non-blocking.
	* Returns FTP_FAILED or FTP_FINISHED or FTP_MOREDATA.
	* See the PHP Manual/function.ftp-nb-continue.html for more details. 
	*
	* @access public
	* @return int
	*/
	function NbContinue()
	{
		return @ftp_nb_continue($this->stream);
	}
	
	/**
	* Returns the size of a file. If an error occurs, of if the file does not exist, -1 
	* is returned. Not all servers support this feature.
	*
	* @access public
	* @return int
	* @param string $filename Remote file name.
	*/
	function Size($filename)
	{
		return @ftp_size($this->stream, $filename);
	}
	
	/**
	* Checks the last-modified time for a file, and returns it as a UNIX timestamp.
	* If an error occurs, or the file does not exist, -1 is returned.
	* Note that not all servers support this feature.
	*
	* @access public
	* @return int
	* @param string $filename Remote file name.
	*/
	function Modified($filename)
	{
		return @ftp_mdtm($this->stream, $filename);
	}
	
	/**
	* Renames (moves) the file on the FTP server.
	*
	* @access public
	* @return bool
	* @param string $source Original file name.
	* @param string $destination New file name.
	*/
	function Rename($source, $destination)
	{
		return @ftp_rename($this->stream, $source, $destination);
	}
	
	/**
	* Deletes the file specified by path from the FTP server.
	*
	* @access public
	* @return bool
	* @param string $path Path of the file to delete.
	*/
	function Delete($path)
	{
		return @ftp_delete($this->stream, $path);
	}
	
	/**
	* Sends an arbitrary command to the FTP server. Returns the server's response as an array
	* of strings. No parsing is performed on the response string, nor does ftp_raw() determine
	* if the command succeeded. 
	*
	* @access public
	* @return array
	* @param string $cmd FTP Command.
	*/
	function Raw($cmd)
	{
		return @ftp_raw($this->stream, $cmd);
	}
	
	/**
	* Request execution of program on FTP server. This may be forbidden for your account.
	*
	* @access public
	* @return bool
	* @param string $cmd Command for execution (program or shell cmd).
	*/
	function Exec($cmd)
	{
		return @ftp_exec($this->stream, $cmd);
	}
	
	/**
	* Sends the command specified by cmd to the FTP server.
	* SITE commands are not standardized, and vary from server to server.
	* They are useful for handling such things as file permissions and group membership.
	*
	* @access public
	* @return bool
	* @param string $cmd FTP server command. Example: "chmod 777 filename.ext".
	*/
	function Site($cmd)
	{
		return @ftp_site($this->stream, $cmd);
	}
	
	/**
	* Closes connection with the FTP server.
	*
	* @access public
	* @return void
	*/
	function Close()
	{
		@ftp_close($this->stream);
	}
	
	/**
	* Opens remote file for reading and returns the file handler.
	*
	* @access public
	* @return resource
	* @param string $filename Remote file name.
	* @param int $mode Transfer mode: FTP_ASCII for text files or FTP_BINARY for others.
	*/
	function FOpenRead($filename, $mode = FTP_ASCII)
	{
		$fp = null;
		$result = @ftp_fget($this->stream, $fp, $filename, $mode);
		if($result) return $fp;
		else return false;
	}
	
	/**
	* Closes remote file open for reading by FTP::FOpenRead() method.
	*
	* @access public
	* @return bool
	* @param resource $fp File pointer.
	*/
	function FCloseRead($fp)
	{
		return fclose($fp);
	}
	
	/**
	* Opens the remote file for writing and returns the file handler.
	*
	* @access public
	* @return resource
	* @param string $filename Remote file name.
	* @param int $mode Transfer mode: FTP_ASCII for text files or FTP_BINARY for others.
	*/
	function FOpenWrite($filename, $mode = FTP_ASCII)
	{
		$local_filename = FTP_TEMP_DIRECTORY.'/'.$filename;
		$this->Get($local_filename, $filename, $mode);
		return fopen($local_filename, 'w+');
	}
	
	/**
	* Closes remote file open for writing by FTP::FOpenWrite() method and saves the changes.
	*
	* @access public
	* @return bool
	* @param string $filename Remote file name.
	* @param resource $fp File pointer.
	* @param int $mode Transfer mode: FTP_ASCII for text files or FTP_BINARY for others.
	*/
	function FCloseWrite($filename, $fp, $mode = FTP_ASCII)
	{
		$local_filename = FTP_TEMP_DIRECTORY.'/'.$filename;
		rewind($fp);
		$result = $this->FPut($filename, $fp, $mode);
		fclose($fp);
		unlink($local_filename);
		return $result;
	}
}
?>